import { Collection } from 'discord.js'
import charactersJSON from './characters.json';
import { z } from 'zod'

export enum Target {
    Self = "self",
    Team = "team",
    OneTeammate = "oneTeammate",
    OneOpponent = "oneOpponent",
    AllOpponents = "allOpponents",
    TrueRandomOpponent = "trueRandomOpponent"
}

const TargetEnum = z.nativeEnum(Target);
type TargetEnum = z.infer<typeof TargetEnum>;

const PotencyStatus = z.object({
    duration: z.number().int(),
    potency: z.number().int()
});
export type PotencyStatus = z.infer<typeof PotencyStatus>;

const Effect = z.object({
    target: TargetEnum,
    accuracy: z.optional(z.number().int()),
    damage: z.optional(z.number().int()),
    heal: z.optional(z.number().int()),
    poison: z.optional(z.number().int()),
    regeneration: z.optional(z.number().int()),
    burn: z.optional(z.number().int()),
    confusion: z.optional(z.number().int()),
    stun: z.optional(z.number().int()),
    resistanceChange: z.optional(z.number().int()),
    accuracyChange: z.optional(z.number().int()),
    speedChange: z.optional(z.number().int()),
    function: z.optional(z.string())
});
type Effect = z.infer<typeof Effect>;

const Skill = z.object({
    name: z.string(),
    accuracy: z.optional(z.number().int()),
    effects: z.array(Effect)
});
type Skill = z.infer<typeof Skill>;

const Character = z.object({
    id: z.number().int(),
    name: z.string(),
    nameShort: z.string(),
    img: z.string(),
    health: z.number().int(),
    speed: z.number().int(),
    skills: z.tuple([Skill, Skill, Skill])
});
export type Character = z.infer<typeof Character>;

const CharacterJSON = z.array(Character);
type CharacterJSON = z.infer<typeof CharacterJSON>;

export const CHARACTERS: Collection<number, Character> = (() => {
    return new Collection(CharacterJSON.parse(charactersJSON).map((element, index) => {return [index, element]}));
})();